/*
 * Copyright 2016 University of Chicago
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef GLOBUS_GRIDFTP_SERVER_S3_STAT_H
#define GLOBUS_GRIDFTP_SERVER_S3_STAT_H 1

#include "globus_gridftp_server_s3_base.h"

enum 
{
    GLOBUS_S3_STAT_COUNT              = 32
};

typedef enum globus_i_stat_type_e
{
    /**
     * For stat(file_only=true, path=/), return dummy info about
     * the root of the s3 service
     */
    GLOBUS_I_S3_STAT_ROOT,
    /**
     * For stat(file_only=true, path="[^/]*"), return info about
     * the named bucket
     */
    GLOBUS_I_S3_STAT_BUCKET,
    /**
     * For stat(file_only=true, path="[^/]*)/(.*)?"), return info about
     * the named object or prefix
     */
    GLOBUS_I_S3_STAT_OBJECT,
    /**
     * For stat(file_only=false, path=/), return info about
     * all buckets the user can access
     */
    GLOBUS_I_S3_LIST_BUCKETS,
    /**
     * For stat(file_only=false, path="[^/]*)/(.*)?"), return info about
     * all objects in the bucket that match the given path prefix (up to the
     * next /)
     */
    GLOBUS_I_S3_LIST_OBJECTS
}
globus_i_stat_type_t;

typedef struct
{
    globus_gfs_operation_t              op;
    /** Bucket Context derived from the session info and the pathname */
    S3BucketContext                     bucket_context;
    /** pointer into pathname that contains the bucket name */
    char                               *bucket_name;
    /** pointer into pathname that contains the key name */
    char                               *key;
    /** Number of retries done so far */
    int                                 retries;
    /** S3 status of the operation */
    S3Status                            status;
    /** Result of the operation */
    globus_result_t                     result;
    /** True if the key is a directory, bucket, placeholder */
    bool                                is_dir;
    /**
     * Type of stat we are attempting
     */
    globus_i_stat_type_t                stat_type;
    /**
     * Stat for a file_only stat
     */
    globus_gfs_stat_t                   stat[GLOBUS_S3_STAT_COUNT];
    /**
     * Number of valid stats in the above array.
     */
    size_t                              stat_count;
    /**
     * Original stat_info for this operation
     */
    globus_gfs_stat_info_t             *stat_info;
    /**
     * marker for resuming the stat of a large directory
     */
    char                               *next_marker;
    /** Copy of the pathname string split into bucket_name and key */
    char                                pathname[];
}
globus_i_s3_stat_t;

extern
void
globus_i_s3_stat_add(
    globus_i_s3_stat_t                 *stat_info,
    globus_gfs_stat_t                  *stat);

extern
void
globus_i_s3_stat_bucket(
    globus_i_s3_stat_t                 *stat_info);

extern
void
globus_i_s3_stat_object(
    globus_i_s3_stat_t                 *stat_info);

extern
void
globus_i_s3_stat_list_objects(
    globus_i_s3_stat_t                 *stat_info);

extern
void
globus_i_s3_stat_root(
    globus_i_s3_stat_t                 *stat_info);

extern
void
globus_i_s3_stat_list_buckets(
    globus_i_s3_stat_t                 *stat_info);

#endif /* GLOBUS_GRIDFTP_SERVER_S3_STAT_H */
// vim: ft=c:
